/*
 * Persistent Systems Private Limited, Pune, India (website - http://www.persistent.co.in)
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that this
 * copyright notice and the following paragraph appears in all
 * copies of this software.
 *
 * DISCLAIMER OF WARRANTY.
 * -----------------------
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
 * IN NO EVENT SHALL THE RIGHTHOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */
#ifndef VOT_CONTROLLER_H
#define VOT_CONTROLLER_H

#include "PParse.h"
#include "VOTGlobal.h"
#include "VOTVOTable.h"
#include "VOTRow.h"


/*
* Main controller class that controls when and where to stop
* the parser. This class communicates with the Progressive parser
* and the parse handler to control when to stop parsing.
* This class controls the execution of the various element handlers.
* It starts & stops the progressive parser, as and when required, 
* depending on the VOTableParser APIs called by the user.
*/

class VOTController
{
public:
	
	// Static Constants
	// These denote the main stops where the parser stops. 
	static const int VOTABLE;
	static const int RESOURCE;
	static const int TABLE;
	static const int ROW;
	static const int DATA;
	static const int RESOURCE_OR_TABLE;
	static const int ACTUAL_TABLEDATA;
	
	/**
	* Default Contructor
	*/
	VOTController();

	/**
	* Destructor
	*/
	~VOTController();

	/**
	* Method to indicate to the controller that start element of required type was found.
	*/
	void gotStartElement(int nextElementType);

	/**
	* Method to indicate to the controller that end element of required type was found.
	*/
	void gotEndElement(int nextElementType);

	/**
	* Method to indicate to the controller where the next parser stop should be.
	*/
	int setNextStopAt(int nextElementType);

	/**
	* Get RESOURCE data.
	*/
	int getResourceData(VOTResource &res);

	/**
	* Get ROW data.
	*/
	int getRowData(VOTRow &row);

	/**
	* Get TABLE data.
	*/
	int getTableData(VOTTable &table);

	/**
	* Get VOTABLE META data.
	*/
	int getVOTableMetaData(VOTVOTable &votable);

	/**
	* Stop at either next RESOURCE or TABLE whichever occures first.
	*/
	int stopAtNextResourceOrTable(int &elementType);
	
	/*
	* Set pointer to Progressive parser and parse handler.
	*/
	int setPParse(PParse * pp);

	/*
	* Get element of given type.
	*/
	int getElement(int elementType);
	
private:
	// tells where to stop next
	int m_nextElementStop ;

	// tells for which end element parser should stop.
	int m_endElementStop ;

	// tells which start element was recieved last.
	int m_gotStartElementType;

	// tells which end element was received last.
	int m_gotEndElementType;

	// tells what current element type is.
	int m_currentElementType;

	// tells what previous element type was.
	int m_previousElementType;

	// tells what previous getElement() was for.
	int m_previousGetElement;

	// indicates if start of Table element stopped the parser
	bool m_startTableStoppedParser;

	// indicates if start of Resource element stopped the parser
	bool m_startResourceStoppedParser;

	// indicates if VOTABLE element is to be parsed from
	// the VOTable or not. It is set to false once the parser current
	// pointer goes beyond VOTABLE element.	
	bool m_getVOTable;
	
	// used to buffer content if Resource or Table is found.
	bool m_getNextTableOrResourceCalled;

	// used to indicate if EOF is reached or not.
	bool m_yetToParse;

	bool m_parsed;

	PParse * m_pparse;
	PParseHandlers * m_pparsehandler;
	StructureHandler * m_handler;

	//////////// private methods /////////////////
	void setPParseHandler(void);
	bool ifResourceAlreadyParsedForVOTable();
	bool ifResourceAlreadyParsed();
	bool ifTableAlreadyParsed();
	void printElementType(int elementType);
	


};

#endif