/*
 * Persistent Systems Private Limited, Pune, India (website - http://www.persistent.co.in)
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that this
 * copyright notice and the following paragraph appears in all
 * copies of this software.
 *
 * DISCLAIMER OF WARRANTY.
 * -----------------------
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
 * IN NO EVENT SHALL THE RIGHTHOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */
#ifndef VO_TABLEPARSER_H
#define VO_TABLEPARSER_H

/* 
* VOTParser.h
*/

#include "PParse.h"
#include <string.h>
#include "VOTVOTable.h"
#include "VOTRow.h"
#include "VOTController.h"

/**
* This is the main class to be used to retrieve all data from
* the VOTable. It contains all the getXXX() and nextXXX() methods
* and methods to start and terminate parsing.
*/

/// This class contains methods for retriving data from VOTable.

class VOTParser
{
public:

	/**
	* Indicates type - TABLE element in the VOTABLE.
	*/
	static const int VOTABLE_TABLE;

	/**
	* Indicates type - RESOURCE element in the VOTABLE.
	*/
	static const int VOTABLE_RESOURCE;
		
	/**
	* Default Constructor
	*/
	VOTParser();
	
	/**
	* Destructor
	*/
	~VOTParser();

	/**
	* Get next element that is either a TABLE or a RESOURCE.
	* Set elementType to 1 to indicate RESOURCE, 2 to indicate TABLE, if either
	* RESOURCE or TABLE are found. 
	* @param elementType The type of element found 1=RESOURCE, 2=TABLE
	* @return VOT_SUCCESS to indicate TABLE or RESOURCE, VOT_ERROR to indicate error, VOT_EOF to indicate end of file.
	*/
	int getNextResourceOrTable(int &elementType);

	/**
	* Go to next RESOURCE in the VOTable. Calling this method will cause the parser
	* to go to the next RESOURCE. This RESOURCE may be nested within or may be at
	* the same nested depth as the previous RESOURCE.
	* @return VOT_SUCCESS to indicate RESOURCE, VOT_ERROR to indicate ERROR, VOT_EOF to indicate EOF.
	*/
	int nextResource(void);

	/**
	* Go to next TABLE in the VOTable. Calling this method will cause the parser
	* to go to the next TABLE without stopping at any  RESOURCE boundaries.
	* @return VOT_SUCCESS to indicate TABLE, VOT_ERROR to indicate ERROR, VOT_EOF to indicate end of file.
	*/
	int nextTable(void);

	/**
	* Go to next ROW in the TABLE. Calling this method will cause the parser
	* to go to the next ROW within the TABLE. It will not cross the TABLE boundary.
	* @return VOT_SUCCESS to indicate ROW, VOT_ERROR to indicate EOF or end of TABLE.
	*/
	int nextRow(void);	

	/**
	* Get TABLE meta-data if the parser currently points to a TABLE. Calling
	* getTable() without calling nextTable() will result in a VOT_ERROR.
	* Note that the table does not contain data, it only contains meta-data.
	* To get actual table data nextRow() and getRow() should be called.
	* @param table VOTTable object to get the TABLE meta data
	* @return VOT_SUCCESS to indicate valid data in table, else VOT_ERROR
	*/
	int getTable(VOTTable &table);

	/**
	* Get RESOURCE data if the parser currently points to a RESOURCE. Calling
	* getResource() without calling nextResource() will result in a VOT_ERROR.
	* @param resource VOTResource object to get the RESOURCE data
	* @return VOT_SUCCESS to indicate valid data in resource, else VOT_ERROR
	*/
	int getResource(VOTResource &resource);

	/**
	* Get VOTable meta-data if the parser currently points to a VOTABLE element. 
	* There is no nextXXX() corresponding to getVOTableMetaData(). You can directly
	* call getVOTableMetaData() to get the VOTable meta-data. But you need to call 
	* this method before you call any nextXXX() function.
	* @param votableMetaData VOTVOTable object to get the VOTABLE meta data
	* @return VOT_SUCCESS to indicate valid data in votableMetaData, else VOT_ERROR
	*/
	int getVOTableMetaData(VOTVOTable &votableMetaData);

	/**
	* Get Row data if the parser currently points to a Row. Calling getRow() 
	* without calling nextRow() will result in a VOT_ERROR.
	* @param row VOTRow object to get the ROW data
	* @return VOT_SUCCESS to indicate valid data in row, else VOT_ERROR.
	*/
	int getRow(VOTRow &row);
	
	/**
	* Initialize parser when the input is from a given file or URL.	
	* Must be called before parsing is initiated.
	* @param filename URL where VOTable is loacted or name of the file to be parsed.
	* @return VOT_SUCCESS to indicate success, else VOT_ERROR.
	*/
	int init(const char *filename);

	/**
	* Initialize parser when the input is from standard input stream.
	* Must be called before parsing is initiated.
	* @return VOT_SUCCESS to indicate success, else VOT_ERROR.
	*/
	int init(void);

	/**
	* Close parser.
	* This should be called for cleanup.
	* @return VOT_SUCCESS to indicate success, else VOT_ERROR.
	*/
	int close(void);

private:
	// Progressive parser
	PParse * m_pparse;

	// Controller object.
	VOTController * m_controller;
};



#endif