/*
 * Persistent Systems Private Limited, Pune, India (website - http://www.pspl.co.in)
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that this
 * copyright notice and the following paragraph appears in all
 * copies of this software.
 *
 * DISCLAIMER OF WARRANTY.
 * -----------------------
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
 * IN NO EVENT SHALL THE RIGHTHOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#ifndef VOT_RESOURCE_H
#define VOT_RESOURCE_H

#include "VOUtils.h"
#include "VOTInfo.h"
#include "VOTCoosys.h"
#include "VOTParam.h"
#include "VOTLink.h"
#include "VOTTable.h"

#include <vector>
using namespace std;

/**
* This class represents &lt;RESOURCE&gt; element in the &lt;VOTABLE&gt;.  
*
* A VOTResource element contains description, list of 'Info' elements,
* list of 'Coosys' elements, list of 'Param' elements, list of
* 'Link' elements. <br>
* <b>Note:</b> It does not list of 'Table' elements, and 'list' of VOTResource 
* elements.
*
*/

/// This class represents &lt;RESOURCE&gt; element in the &lt;VOTABLE&gt;

//Date created - 27 May 2002

class VOTResource {

	public:
		/**
		* Default constructor
		*/
		VOTResource();

		/**
		* Destructor
		*/
		~VOTResource();

		/**
		* Assignment operator overloaded.
		*/
		VOTResource operator=(const VOTResource &resource);

		/**
		* Copy constructor.
		*/
		VOTResource(const VOTResource &resource);
		
		/**
		* Get the 'description' of VOTResource.
		*/
		int getDescription(char * &desc );

		/**
		* Get the 'ID'.
		*/
		int getID(char * &ID );

		/**
		* Get the 'name'.
		*/
		int getName(char * &name );

		/**
		* Get the 'type'.
		*/
		int getType(resource_type &value);

		/**
		* Get the number of 'Info' elements.
		*/
		int getNumOfInfos(int &numOfElements);

		/**
		* Get the 'Info' element, given the index.
		* Index starts from 0.
		*/
		int getInfo(VOTInfo &info, int index);
		
		/**
		* Get the number of 'Coosys' elements.
		*/
		int getNumOfCoosys(int &numOfElements );

		/**
		* Get the 'Coosys' element, given the index.
		* Index starts from 0.
		*/
		int getCoosys(VOTCoosys &coosys, int index );

		/**
		* Get the number of 'Param' elements.
		*/
		int getNumOfParams(int &numOfElements );

		/**
		* Get the 'Param' element, given the index.
		* Index starts from 0.
		*/
		int getParam(VOTParam &param, int index );

		/**
		* Get the number of 'Link' elements.
		*/
		int getNumOfLinks(int &numOfElements );

		/**
		* Get the 'Link' element, given the index.
		* Index starts from 0.
		*/
		int getLink(VOTLink &link, int index );

		/**
		* Get the number of 'VTable' elements.
		*/
		int getNumOfTables(int &numOfElements );

		/**
		* Get the 'VTable' element, given the index.
		* Index starts from 0.
		*/
//		int getTable(VOTTable &table, int index );

		/**
		* Get nested depth of resource.
		* Nested depth is 0 for a RESOURCE if it is an immediate child of VOTABLE element.
		* For a RESOURCE within a RESOURCE the nested depth will be greater than 0, depending
		* on its distance from the VOTABLE element.
		* 
		*/
		int getNestedDepth(int &depth);

		/*
		 * Reinitializes all the data structures in the class
		 */
		void reset(void);

		int setDesc(char * infoValue );
		int setID(char * str);
		int setName(char * str);
		int setType(resource_type t);
		int setNestedDepth(int depth);

		int addInfo(VOTInfo info );
		int addCoosys(VOTCoosys coosys);
		int addParam(VOTParam param );
		int addLink(VOTLink link );
		
	private: 
		
		char * m_desc;
		char * m_ID;
		char * m_name;
		int m_nestedDepth;
		resource_type m_type;

		vector <VOTInfo> m_infoList;
		vector <VOTCoosys> m_coosysList;
		vector <VOTParam> m_paramList;
		vector <VOTLink> m_linkList;			
		
		void cleanup(void);
		void makecopy(const VOTResource &r);
		void init(void);
};

#endif
